@@ -11,11 +11,11 @@ files or a direct python definition file.
 __docformat__ = "restructuredtext en"
 
 import sys
+import types
 from os import listdir
 from os.path import exists, join, splitext, basename
 from warnings import warn
 
-from logilab.common import attrdict
 from logilab.common.testlib import mock_object
 from logilab.common.textutils import get_csv
 from logilab.common.modutils import modpath_from_file, cleanup_sys_modules
@@ -78,10 +78,10 @@ class PyFileReader(object):
     def read_file(self, filepath):
         self._current_file = filepath
         try:
-            modname, fdata = self._loaded[filepath]
+            modname, module = self._loaded[filepath]
         except KeyError:
-            modname, fdata = self.exec_file(filepath)
-        for name, obj in fdata.items():
+            modname, module = self.exec_file(filepath)
+        for name, obj in vars(module).items():
             if name.startswith('_'):
                 continue
             try:
@@ -94,7 +94,7 @@ class PyFileReader(object):
     def import_erschema(self, ertype, schemamod=None, instantiate=True):
         warn('import_erschema is deprecated, use explicit import once schema '
              'is turned into a proper python module (eg not expecting '
-             'predefined context in globals)', DeprecationWarning)
+             'predefined context in globals)', DeprecationWarning, stacklevel=3)
         try:
             erdef = self.loader.defined[ertype]
             name = hasattr(erdef, 'name') and erdef.name or erdef.__name__
@@ -107,7 +107,7 @@ class PyFileReader(object):
 
     def exec_file(self, filepath):
         try:
-            modname = '.'.join(modpath_from_file(filepath))
+            modname = '.'.join(modpath_from_file(filepath, self.loader.extrapath))
             doimport = True
         except ImportError:
             warn('module for %s can\'t be found, add necessary __init__.py '
@@ -124,11 +124,6 @@ class PyFileReader(object):
             module = sys.modules[modname]
             # NOTE: don't test raw equality to avoid .pyc / .py comparisons
             assert module.__file__.startswith(filepath), (filepath, module.__file__)
-            if not isinstance(module, attrdict):
-                # module imported by regular import, we've to provide .items for
-                # bw compat until we can use proper module objects everywhere
-                # XXX we may miss some deprecation warning here
-                module.items = lambda : vars(module).items()
         else:
             # XXX until bw compat is gone, put context into builtins to allow proper
             # control of deprecation warning
@@ -167,13 +162,16 @@ class PyFileReader(object):
                         if pname in fglobals or not pname in self.context:
                             # imported
                             continue
-                        warn('%s: please explicitly import %s'
-                             % (filepath, pname), DeprecationWarning)
+                        warn('%s: please explicitly import %s (%s)'
+                             % (filepath, pname, name), DeprecationWarning)
             for key in self.context:
                 fglobals.pop(key, None)
             fglobals['__file__'] = filepath
-            module = attrdict(fglobals)
+            module = types.ModuleType(str(modname))
+            module.__dict__.update(fglobals)
             sys.modules[modname] = module
+            if package:
+                setattr(sys.modules[package], modname.split('.')[-1], module)
         self._loaded[filepath] = (modname, module)
         return self._loaded[filepath]
 
@@ -192,6 +190,7 @@ class SchemaLoader(object):
     set of files
     """
     schemacls = schemamod.Schema
+    extrapath = None
 
     def load(self, directories, name=None,
              register_base_types=True, construction_mode='strict',
