@@ -11,6 +11,7 @@ import warnings
 from copy import deepcopy
 from decimal import Decimal
 
+from logilab.common import attrdict
 from logilab.common.decorators import cached
 from logilab.common.compat import sorted
 from logilab.common.interface import implements
@@ -197,6 +198,7 @@ class EntitySchema(ERSchema):
     # they may be missing from schemas obtained by pyro
     _specialized_type = None
     _specialized_by = []
+
     def __init__(self, schema=None, rdef=None, *args, **kwargs):
         super(EntitySchema, self).__init__(schema, rdef, *args, **kwargs)
         if rdef is not None:
@@ -349,7 +351,8 @@ class EntitySchema(ERSchema):
         rschema = self.subject_relation(rtype)
         assert rschema.is_final(), (self.type, rtype)
         objtypes = rschema.objects(self.type)
-        assert len(objtypes) == 1
+        assert len(objtypes) == 1, (self.type, str(rtype),
+                                    [str(ot) for ot in objtypes])
         return objtypes[0]
 
     def subjrproperty(self, rtype, prop):
@@ -931,6 +934,8 @@ class Schema(object):
     __implements__ = ISchema
     entity_class = EntitySchema
     relation_class = RelationSchema
+    # relation that should not be infered according to entity type inheritance
+    no_specialization_inference = ()
     # __hashmode__ is a evil hack to support schema pickling
     # it should be set to 'pickle' before pickling is done and reset to None
     # once it's done
@@ -1072,25 +1077,28 @@ class Schema(object):
         for rschema in eschema._obj_relations.values():
             for subjtype in rschema.subjects(etype):
                 self.del_relation_def(subjtype, rschema, eschema)
+        if eschema.specializes():
+            eschema.specializes()._specialized_by.remove(eschema)
+        if eschema.specialized_by():
+            raise Exception("can't remove entity type %s used as parent class by %s" %
+                            (eschema, ','.join(str(et) for et in eschema.specialized_by())))
         del self._entities[etype]
 
     def infer_specialization_rules(self):
-        # XXX
-        class XXXRelationDef:
-            infered = True
         for rschema in self.relations():
-            if rschema.is_final():
+            if rschema.is_final() or rschema in self.no_specialization_inference:
                 continue
             for subject, object in rschema.rdefs():
                 subjeschemas = [subject] + subject.specialized_by(recursive=True)
                 objeschemas = [object] + object.specialized_by(recursive=True)
                 for subjschema in subjeschemas:
                     for objschema in objeschemas:
-                        subjobj = (subjschema, objschema)
                         # don't try to add an already defined relation
-                        if subjobj in rschema.rdefs():
+                        if rschema.has_rdef(subjschema, objschema):
                             continue
-                        rschema.update(subjschema, objschema, XXXRelationDef)
+                        rdef = attrdict(rschema.rproperties(subject, object))
+                        rdef['infered'] = True
+                        rschema.update(subjschema, objschema, rdef)
 
     def remove_infered_definitions(self):
         """remove any infered definitions added by
